<?php

/* imap.php: core IMAP functions :) 
*
    Copyright (C) 2002,2003,2004  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

* $Id: imap.php,v 1.265 2004/08/18 05:23:35 sailfrog Exp $
*/

/**
 *  get IMAP capability response
 *
 *  @param  $imap_stream    resource    connection to the imap server
 *  @return                 string      the imap CAPABILITY response or an error string
 *
 */
function hm_get_caps($imap_stream) {
    $str = $_SESSION['str'];
    $_SESSION['counter']++;
    $pre = 'A000'.$_SESSION['counter'];
    $pre_len = strlen($pre);
    $result = array();
    $command = "$pre CAPABILITY\r\n";
    $n = -1;
    $caps_string = $str[200];
    fputs($imap_stream, $command);
    do {
        $n++;
        $result[$n] = fgets($imap_stream, 1024);
        if (stristr($result[$n], '* CAPABILITY')) {
            $caps_string = $result[$n];
        }
    } while (substr($result[$n], 0, $pre_len) != $pre);
    return $caps_string;
}

/**
 *  login to imap
 *
 *  @param  $user       string          imap username
 *  @param  $encpass    string          encrypted password
 *  @param  $config     array           configuration values
 *  @param  $hasty_key  string          The encryption key
 *  @param  $response   boolean         if true, puts the login string in the session
 *  @return             resource|false  the imap connection on success 
 *
 */    
function hm_imap_login ($user, $encpass, $config, $hasty_key, $response=false, 
                        $debug=false) {

    // add some debug and checks for these config options
    $server = $config['imap_server'];
    $port = $config['imap_port'];
    $ssl = $config['imap_ssl'];

    $result = array();
    $pass = hmw_crypt($encpass, $config, 'decrypt', $hasty_key);
    if (!$pass && $debug) {
        echo '<font color="blue">Could not get the encrypted password.</font><br>',
             'Make sure the encryption key is setup as described in',
             'the INSTALL file<br>';
    }
    if ($ssl == 'ssl') {
        $server = 'ssl://'.$server;
    }
    if ($ssl == 'tls') {
        $server = 'tls://'.$server;
    }
    if ($debug) {
        echo 'trying to connect to the IMAP server at this address:<font color="blue"> ',
              $server,'</font><br>';
        if ($ssl == 'ssl' || $ssl == 'tls') {
            echo 'connecting over ',$ssl,' to port # <font color="blue">',$port,
                 '</font><br>';
        }
        else {
            echo 'connection on a normal socket on port # <font color="blue">',$port,
                 '</font><br>';
        } 
    }
    if ($is = @fsockopen($server, $port, $errorno, $errorstr, 30)) {
        if (is_resource($is)) {
            if ($debug) {
                echo '<b>successfully opened socket to imap server</b><br>';
            }
            $result[0] = fgets($is, 1024);
            if (isset($config['imap_auth']) && strtolower($config['imap_auth'])
                == 'cram-md5') {
                fputs ($is, 'A0001 AUTHENTICATE CRAM-MD5'."\r\n");
                $response = fgets($is, 1024);
                $challenge = base64_decode(substr($response, 4));
                $key .= str_repeat(chr(0x00), (64-strlen($key)));
                $ipad = str_repeat(chr(0x36), 64);
                $opad = str_repeat(chr(0x5c), 64);
                $digest = bin2hex(pack("H*", md5(($key ^ $opad).
                          pack("H*", md5(($key ^ $ipad).$challenge)))));
                $challenge_response = base64_encode($user.' '.$digest);
                fputs($is, $challenge_response."\r\n");
                $n = 1;
                $result[$n] = fgets($is, 1024);
            }
            else {
                fputs($is, 'A0001 LOGIN "'.str_replace('"', '\"', $user).
                      '" "'.str_replace('"', '\"', $pass). "\"\r\n");
                $n = 0;
                do {
                    $n++;
                    $result[$n] = fgets($is, 1024);
                } while (substr($result[$n], 0, 5) != 'A0001');
            }
            if (stristr($result[$n], 'A0001 OK')) {
                if ($response) {
                    $_SESSION['login_string'] = $result[0];
                }
                $_SESSION['counter'] = 1;
                if ($debug) {
                    echo '<br><br><br><center>',
                         '<br>You have successfully logged into the IMAP server!<br>
                          <b>You can now disable the login_debug in hastymail.conf</b>',
                         '</center>';
                    exit;
                }
                return $is;
            }
            else {
                if ($debug) {
                    echo 'The imap server did not like us, it said: <font color="blue">',
                         $result[$n],'</font><br>';
                    exit;
                }
                return false;
            }
        }
        else {
            if ($debug) {
                echo 'could not connect to the imap server: <font color="blue">',
                      '<br>Error number:',$errorno,
                      '<br>Error name:',$errorstr,'</font><br>';
                exit;
            }
            return false;
        }
    }
    else {
        if ($debug) {
            echo 'could not connect to the imap server: <font color="blue">',
                  $errorno, $errorstr,'</font><br>';
            exit;
        }
        return false;
    }
}

/**
 *  select a mailbox
 *
 *  @param  $imap_stream    resource    connection to imap
 *  @param  $mailbox        string      the mailbox to select
 *  @return                 void
 *
 */
function hm_select_mailbox ($imap_stream, $mailbox, $die=false) {
    $_SESSION['counter']++;
    $str = $_SESSION['str'];
    $pre = 'A000'.$_SESSION['counter'];
    $pre_len = strlen($pre);
    $command = "$pre SELECT \"".urldecode($mailbox)."\"\r\n";
    fputs($imap_stream, $command);
    $n = -1;
    $total = false;
    $uidv  = false;
    do {
        $n++;
        $result[$n] = fgets($imap_stream, 1024);
        $parts = hm_imap_parse_line(str_replace(']',' ]', $result[$n]), $imap_stream);
        $count = count($parts);
        for ($i=0;$i<$count;$i++) {
            if (stristr($parts[$i], 'EXISTS') && isset($parts[$i - 1]) &&
                is_numeric($parts[$i - 1])) {
                $total = $parts[$i - 1];
            }
            if (stristr($parts[$i], 'UIDVALIDITY') && isset($parts[$i + 1]) &&
                is_numeric($parts[$i] + 1)) {
                $uidv = $parts[$i + 1];
            }
        }
    }
    while (substr($result[$n], 0, $pre_len) != $pre);
    if (!stristr($result[$n], "$pre OK")) {
        if ($die) {
            hm_check_errors($result[$n], "$pre OK", $command);
        }        
        $_SESSION['missing_folders_flag'] = 1;
        $_SESSION['missing_folders'][] = urldecode($mailbox);
        $mailbox = 'INBOX';
        hm_select_mailbox ($imap_stream, $mailbox, true);
    }
    if (isset($_SESSION['message_count_select'][$mailbox]) && $total &&
        $_SESSION['message_count_select'][$mailbox] != $total) {
        $_SESSION['changed'][$mailbox] = 1;
    }
    if (isset($_SESSION['valid'][$mailbox]) && $uidv &&
        $_SESSION['valid'][$mailbox] != $uidv) {
        $_SESSION['changed'][$mailbox] = 1;
    }
    $_SESSION['message_count_select'][$mailbox] = $total;
    $_SESSION['valid'][$mailbox] = $uidv;
    return $mailbox;
}

/**
 *  get the unseen status for all selected mailboxes
 *
 *  @param  $imap_stream    resource    conneciton to imap
 *  @param  $folders        array       array of folder names
 *  @return                 array       keys are folders, values are unseen count
 *
 */
function hm_unseen_status ($imap_stream, $folders) {
    $unseen = array();
    $uidnext = array();
    $valid = array();
    $count = array();
    $ans = false;
    if (isset($_SESSION['settings']['new_mail_folders'])) {
        $folders = $_SESSION['settings']['new_mail_folders'];
    }
    else {
        $folders = array_diff($_SESSION['folders'], $_SESSION['nosel_folders']);
    }
    $command = '';
    if (!empty($folders)) {
        foreach ($folders as $folder) {
            $_SESSION['counter']++;
            $pre = 'A000'.$_SESSION['counter'];
            $pre_len = strlen($pre);
            $command .= "$pre STATUS \"".trim($folder).
                    "\" (UNSEEN MESSAGES UIDNEXT)\r\n";
        }
    }
    else {
        return array();
    }
    fputs($imap_stream, $command);
    $l = -1;
    do {
        $l++;
        $result[$l] = fgets($imap_stream, 1024);
        $parts = hm_imap_parse_line(str_replace(')', ' )', $result[$l]), $imap_stream);
        $test = strtoupper($parts[0]);
        if ($test == 'STATUS') {
            $folder = $parts[1];
            $parts = array_slice($parts, 3);
            foreach ($parts as $index => $value) {
                switch(strtoupper($value)) {
                    case 'MESSAGES':
                        if (isset($parts[$index + 1]) && is_numeric($parts[$index + 1])) {
                            $count[$folder] = $parts[$index + 1];
                            if (isset($_SESSION['message_count'][$folder]) &&
                                $_SESSION['message_count'][$folder] != $count[$folder]) {
                                $_SESSION['changed'][$folder] = 1;
                            }
                        }
                        break;
                    case 'UNSEEN':
                        if (isset($parts[$index + 1]) && is_numeric($parts[$index + 1])) {
                            $unseen[$folder] = $parts[$index + 1];
                        }
                        break;
                    case 'UIDNEXT':
                        if (isset($parts[$index + 1]) && is_numeric($parts[$index + 1])) {
                            $uidnext[$folder] = $parts[$index + 1];
                            if (isset($_SESSION['uidnext'][$folder]) &&
                                $_SESSION['uidnext'][$folder] != $uidnext[$folder]) {
                                $_SESSION['changed'][$folder] = 1;
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
    } while (substr($result[$l], 0, $pre_len) != $pre);
    $_SESSION['uidnext'] = $uidnext;
    $_SESSION['unseen'] = $unseen;
    $_SESSION['message_count'] = $count;
    return $unseen;
}

/**
 *  terminates the imap connection
 *
 *  @param      $imap_stream    resource    conneciton to imap
 *  @return                     void
 *
 */
function hm_imap_logout ($imap_stream) {
    $command = "A1000 LOGOUT\r\n";
    fputs($imap_stream, $command);
    $result = fgets($imap_stream, 1024);
}

/**
 *  break up a line 
 *
 *  @param      $line               string          first line of imap response
 *  @param      $imap_stream        resource        connection to imap
 *  @return                         array           parsed response
 *
 */
function hm_imap_parse_line($line, $imap_stream) {
    $parts = array();
    while ($line) {
        $chunk = false;
        switch ($line{0}) {
            case "\r":
            case "\n":
                $line = false;
                break;
            case '"':
                if (preg_match("/^(\"[^\"\\\]*(?:\\\.[^\"\\\]*)*\")/",
                    $line, $matches)) {
                    $chunk = substr($matches[1], 1, -1);
                }
                $line = substr($line, strlen($chunk) + 2);
                break;
            case '*':
            case '[':
            case ']':
                $line = substr($line, 1);
                break;
            case '(':
            case ')':
                $chunk = $line{0};
                $line = substr($line, 1);
                break;
            case ' ':
                $line = substr($line, 1);
                break;
            case '{':
                $end = strpos($line, '}');
                if ($end !== false) {
                    $literal_size  = substr($line, 1, ($end - 1));
                }
                $chunk = hm_read_literal($imap_stream, $literal_size);
                $line = false;
                break;
            default:
                if (strpos($line, ' ') !== false) {
                    $chunk = substr($line, 0, strpos($line, ' '));
                    $line = substr($line, strlen($chunk));
                }
                else {
                    $chunk = $line;
                    $line = false;
                }
                break;
        }
        if (is_string($chunk)) {
            $parts[] = $chunk;
        }
    }
    return $parts;
}

/**
 *  read in literal data
 *
 *  @param      $imap_stream        resource        connection to imap
 *  @param      $size               int             amount of literal data
 *  @return                         array           parsed response
 *
 */
function hm_read_literal($imap_stream, $size) {
    $literal_data = fgets($imap_stream, 2048);
    while (substr($literal_data , -1) != "\n") {
        $literal_data .= fgets($imap_stream, 2048);
    }
    while (strlen($literal_data) < $size) {
        $literal_data .= fgets($imap_stream, 2049);
        while (substr($literal_data , -1) != "\n") {
            $literal_data .= fgets($imap_stream, 2048);
        } 
    }
    return $literal_data;
}

?>
